==> herbert-0.4.0.jar: miner/buggy.clj <==
;; Junk removed from herbert.clj -- I might want to come back to it again.


;; want to compile constraint a macro-expansion
;; BUGGY

;; (quote [(a int) {:a (a)}]) clojure.lang.Cons
;; #<herbert$con-rule$ff__7792 miner.herbert$con-rule$ff__7792@6751406> miner.herbert$con-rule$ff__7792
;; IllegalArgumentException No matching ctor found for class miner.herbert$con-rule$ff__7792  clojure.lang.Reflector.invokeConstructor (Reflector.java:163)

;;http://stackoverflow.com/questions/11191992/functions-with-closures-and-eval-in-clojure/11287181#11287181
;;http://dev.clojure.org/jira/browse/CLJ-1206 "eval of closures"

;; SEE http://stackoverflow.com/questions/7852351/clojure-macro-to-generate-functions


;; SEM FIXME -- maybe you can make it work using intern or gensym a var
;; The problem seems to be that the "literal" fn does work when crossing over from the macro
;; creation to the runtime evaluation.  A var might work-around that.

(defmacro WORKS-conf? [con x]
  (list (list 'con-rule con) x))

;; fails with the ctor bug
(defmacro conf? [con x]
  (let [cf (con-rule con)]
    (with-local-vars [cff cf]
      (list cff x))))

#_(defmacro conf? [con x]
  (println con (type con))
  (let [conf (con-rule con)]
    (println conf (type conf))
    `(let [res# (~conf ~x)]
       (when (sp/success? res#)
         (with-meta (:b res#) {::constraint ~con})))))

;; BROKEN
#_ (defmacro conf? [con x]
  (println con (type con))
  (let [conf (fn [c] ((con-rule con) c))]
    (println conf (type conf))
    `(let [res# (~conf ~x)]
       (when (sp/success? res#)
         (with-meta (:b res#) {::constraint ~con})))))


;; WORKS, but I want a macro!
(defn fn-conf? [con x]
  (println con (type con))
  (let [conf (con-rule con)]
    (println conf (type conf))
    (let [res (conf x)]
       (when (sp/success? res)
         (with-meta (:b res) {::constraint con})))))

;; WORKS but strange
(defn vconf? [con x]
  (with-local-vars [conf (con-rule con)]
     (let [res (conf x)]
       (when (sp/success? res)
         (with-meta (:b res) {::constraint con})))))


(defmacro lconf? [con x]
  (println con (type con))
  (def fcf (con-rule con))
    (println fcf (type fcf))
    `(let [res# (#'fcf ~x)]
       (when (sp/success? res#)
         (with-meta (:b res#) {::constraint ~con}))))


#_ (definline conf? [con x]
  `(~(con-rule con) ~x))


;; Bad
#_ (defmacro zconf? [con x]
  (let [vname (gensym)]
    `(do (def ~vname ~(con-rule con))
         (let [res# (~vname ~x)]
           (when (sp/success? res#)
             (with-meta (:b res#) {::constraint ~con}))))))

==> herbert-0.5.6-20131021.153437-1.jar: miner/buggy.clj <==
;; Junk removed from herbert.clj -- I might want to come back to it again.




;; UGLY and unfinished -- don't use this
(defmacro conf? [con x]
  (println "conf?" con (type con))
  (let [dcon (if (and (seq? con) (= (first con) 'quote)) (second con) con)
        fcf (conform dcon)]
    (println "  dc" dcon (type dcon))
    (println "  cf " fcf (type fcf))
    `(let [res# (fcf ~x)]
       (when res#  (with-meta res# {::constraint ~con})))))




;; want to compile constraint a macro-expansion
;; BUGGY

;; (quote [(a int) {:a (a)}]) clojure.lang.Cons
;; #<herbert$con-rule$ff__7792 miner.herbert$con-rule$ff__7792@6751406> miner.herbert$con-rule$ff__7792
;; IllegalArgumentException No matching ctor found for class miner.herbert$con-rule$ff__7792  clojure.lang.Reflector.invokeConstructor (Reflector.java:163)

;;http://stackoverflow.com/questions/11191992/functions-with-closures-and-eval-in-clojure/11287181#11287181
;;http://dev.clojure.org/jira/browse/CLJ-1206 "eval of closures"

;; SEE http://stackoverflow.com/questions/7852351/clojure-macro-to-generate-functions


;; SEM FIXME -- maybe you can make it work using intern or gensym a var
;; The problem seems to be that the "literal" fn does work when crossing over from the macro
;; creation to the runtime evaluation.  A var might work-around that.

(defmacro WORKS-conf? [con x]
  (list (list 'con-rule con) x))

;; fails with the ctor bug
(defmacro conf? [con x]
  (let [cf (con-rule con)]
    (with-local-vars [cff cf]
      (list cff x))))

#_(defmacro conf? [con x]
  (println con (type con))
  (let [conf (con-rule con)]
    (println conf (type conf))
    `(let [res# (~conf ~x)]
       (when (sp/success? res#)
         (with-meta (:b res#) {::constraint ~con})))))

;; BROKEN
#_ (defmacro conf? [con x]
  (println con (type con))
  (let [conf (fn [c] ((con-rule con) c))]
    (println conf (type conf))
    `(let [res# (~conf ~x)]
       (when (sp/success? res#)
         (with-meta (:b res#) {::constraint ~con})))))


;; WORKS, but I want a macro!
(defn fn-conf? [con x]
  (println con (type con))
  (let [conf (con-rule con)]
    (println conf (type conf))
    (let [res (conf x)]
       (when (sp/success? res)
         (with-meta (:b res) {::constraint con})))))

;; WORKS but strange
(defn vconf? [con x]
  (with-local-vars [conf (con-rule con)]
     (let [res (conf x)]
       (when (sp/success? res)
         (with-meta (:b res) {::constraint con})))))


(defmacro lconf? [con x]
  (println con (type con))
  (def fcf (con-rule con))
    (println fcf (type fcf))
    `(let [res# (#'fcf ~x)]
       (when (sp/success? res#)
         (with-meta (:b res#) {::constraint ~con}))))


#_ (definline conf? [con x]
  `(~(con-rule con) ~x))


;; Bad
#_ (defmacro zconf? [con x]
  (let [vname (gensym)]
    `(do (def ~vname ~(con-rule con))
         (let [res# (~vname ~x)]
           (when (sp/success? res#)
             (with-meta (:b res#) {::constraint ~con}))))))
